@@ -5,26 +5,33 @@ use warnings;
 
 use File::Basename ();
 use File::Spec;
+use Carp;
 
 use base 'Exporter';
 use vars '@EXPORT_OK';
 
-@EXPORT_OK = qw(path dirname read_file_content read_glob_content);
+@EXPORT_OK = qw(path dirname read_file_content read_glob_content open_file);
 
 sub path    { File::Spec->catfile(@_) }
 sub dirname { File::Basename::dirname(@_) }
 
+sub open_file {
+    my ($mode, $filename) = @_;
+    require Dancer::Config;
+    my $charset = Dancer::Config::setting('charset');
+    length($charset || '')
+      and $mode .= ":encoding($charset)";
+    open(my $fh, $mode, $filename)
+      or croak "$! while opening '$filename' using mode '$mode'";
+    return $fh;
+}
+
 sub read_file_content {
     my ($file) = @_;
     my $fh;
 
-    require Dancer::Config;
-    my $charset = Dancer::Config::setting('charset');
-
-    my $open_flag = '<';
-    $open_flag = '<:encoding(UTF-8)' 
-        if $charset eq 'UTF-8';
-    if ($file && open($fh, $open_flag, $file)) {
+    if ($file) {
+        $fh = open_file('<', $file);
         return read_glob_content($fh);
     }
     else {
@@ -37,7 +44,7 @@ sub read_glob_content {
 
     # we don't want to do that as we'll encode the stuff later
     # binmode $fh;
-    
+
     my @content = <$fh>;
     close $fh;
     my $content = join("", @content);
@@ -68,6 +75,14 @@ use additional modules.
 
 =head1 SUBROUTINES/METHODS
 
+=head2 open_file
+
+    use Dancer::FileUtils 'open_file';
+    my $fh = open_file('<', $file) or die $message;
+
+Calls open and returns a filehandle. Takes in account the 'charset' setting to
+open the file in the proper encoding.
+
 =head2 path
 
     use Dancer::FileUtils 'path';
@@ -101,9 +116,9 @@ in case it failed to open the file.
     open my $fh, '<', $file or die "$!\n";
     my $content = read_glob_content($fh);
 
-Same as I<read_file_content>, only it accepts a file handler.
+Same as I<read_file_content>, only it accepts a file handle.
 
-Returns the content and B<closes the file handler>.
+Returns the content and B<closes the file handle>.
 
 =head1 EXPORT
 
