@@ -12,14 +12,10 @@ use Dancer::Request;
 use Dancer::Response;
 use Dancer::Serializer;
 use Dancer::Config 'setting';
-use Dancer::FileUtils qw(path dirname read_file_content);
+use Dancer::FileUtils qw(path dirname read_file_content open_file);
 use Dancer::SharedData;
 use Dancer::Logger;
-use MIME::Types;
-
-BEGIN {
-    MIME::Types->new(only_complete => 1);
-}
+use Dancer::MIME;
 
 sub render_file {
     return get_file_response();
@@ -119,7 +115,7 @@ sub get_action_response {
     }
 
     # redirect immediately - skip route execution
-    if (my $status = Dancer::Response::status) {
+    if (my $status = Dancer::Response->status) {
         if ($status == 302 || $status == 301) {
             return serialize_response_if_needed(Dancer::Response->current);
         }
@@ -147,7 +143,7 @@ sub get_action_response {
 sub serialize_response_if_needed {
     my ($response) = @_;
     $response = Dancer::Serializer->process_response($response)
-      if Dancer::App->current->setting('serializer') && $response->{content};
+      if Dancer::App->current->setting('serializer') && $response->content();
     return $response;
 }
 
@@ -165,14 +161,14 @@ sub get_file_response_for_path {
     $status ||= 200;
 
     if (-f $static_file) {
-        open my $fh, "<", $static_file;
+        my $fh = open_file('<', $static_file);
         binmode $fh;
 
-        Dancer::Response::status($status);
-        Dancer::Response::content_type(
-            get_mime_type($static_file));
-        Dancer::Response::content($fh);
-        return Dancer::Response->current;
+        return Dancer::Response->new(
+            status  => $status,
+            headers => ['Content-Type' => get_mime_type($static_file)],
+            content => $fh
+        );
     }
     return;
 }
@@ -181,18 +177,10 @@ sub get_file_response_for_path {
 
 sub get_mime_type {
     my ($filename) = @_;
-    my @tokens = reverse(split(/\./, $filename));
-    my $ext = $tokens[0];
+    my ($ext) = reverse(split(/\./, $filename));
 
-    # first check user configured mime types
-    my $mime = Dancer::Config::mime_types($ext);
-    return $mime if defined $mime;
-
-    # user has not specified a mime type, so ask MIME::Types
-    $mime = MIME::Types->new(only_complete => 1)->mimeTypeOf($ext);
-
-    # default to text/plain
-    return defined($mime) ? $mime : 'text/plain';
+    my $mime = Dancer::MIME->instance();
+    return $mime->mime_type_for($ext);
 }
 
 # set of builtin templates needed by Dancer when rendering HTML pages
