@@ -2,6 +2,7 @@ package Dancer::Template::TemplateToolkit;
 
 use strict;
 use warnings;
+use Carp;
 use Dancer::Config 'setting';
 use Dancer::ModuleLoader;
 use Dancer::FileUtils 'path';
@@ -13,13 +14,16 @@ my $_engine;
 sub init {
     my ($self) = @_;
 
-    die "Template is needed by Dancer::Template::TemplateToolkit"
+    croak "Template is needed by Dancer::Template::TemplateToolkit"
       unless Dancer::ModuleLoader->load('Template');
 
+    my $charset = setting('charset') || '';
+    my @encoding = length($charset) ? ( ENCODING => $charset ) : ();
+
     my $tt_config = {
-        ANYCASE   => 1,
-        ABSOLUTE  => 1,
-        ENCODING  => 'utf8',
+        ANYCASE  => 1,
+        ABSOLUTE => 1,
+        @encoding,
         %{$self->config},
     };
 
@@ -41,11 +45,13 @@ sub init {
 
 sub render {
     my ($self, $template, $tokens) = @_;
-    die "'$template' is not a regular file"
+    croak "'$template' is not a regular file"
       if !ref($template) && (!-f $template);
 
     my $content = "";
-    $_engine->process($template, $tokens, \$content) or die $_engine->error;
+    my $charset = setting('charset') || '';
+    my @options = length($charset) ? ( binmode => ":encoding($charset)" ) : ();
+    $_engine->process($template, $tokens, \$content, @options) or croak $_engine->error;
     return $content;
 }
 
